from src.cli_report.report import main
from unittest.mock import patch, mock_open

start_log = ('SVF2018-05-24_12:02:58.917\n'
             'VBM2018-05-24_12:00:00.000')
end_log = ('VBM2018-05-24_12:01:12.434\n'
           'SVF2018-05-24_12:04:03.332')
abbreviations_txt = ('SVF_Sebastian Vettel_FERRARI\n'
                     'VBM_Valtteri Bottas_MERCEDES')


def test_mock_open_file():
    with patch("builtins.open", mock_open(read_data=start_log)) as mock_file:
        assert open('start.log').readlines() == ['SVF2018-05-24_12:02:58.917\n', 'VBM2018-05-24_12:00:00.000']
        mock_file.assert_called_once_with('start.log')
    with patch("builtins.open", mock_open(read_data=end_log)) as mock_file:
        assert open('end.log').readlines() == ['VBM2018-05-24_12:01:12.434\n', 'SVF2018-05-24_12:04:03.332']
        mock_file.assert_called_once_with('end.log')
    with patch("builtins.open", mock_open(read_data=abbreviations_txt)) as mock_file:
        assert open('abbreviations.txt').readlines() == ['SVF_Sebastian Vettel_FERRARI\n',
                                                         'VBM_Valtteri Bottas_MERCEDES']
        mock_file.assert_called_once_with('abbreviations.txt')


def test_main_function(tmpdir, capsys):
    start_log_path = tmpdir.join("start.log")
    start_log_path.write(start_log)
    end_log_path = tmpdir.join("end.log")
    end_log_path.write(end_log)
    abbreviations_txt_path = tmpdir.join("abbreviations.txt")
    abbreviations_txt_path.write(abbreviations_txt)
    assert start_log_path.read() == start_log
    assert end_log_path.read() == end_log
    assert abbreviations_txt_path.read() == abbreviations_txt
    folder_path = str(tmpdir)
    with patch("sys.argv", ['main', '--files', folder_path]):
        main()
    captured = capsys.readouterr()
    assert captured.out == ("Sebastian Vettel     | FERRARI                   | 1:04.415\n"
                            "Valtteri Bottas      | MERCEDES                  | 1:12.434\n")
    with patch("sys.argv", ['main', '--files', folder_path, '--desc']):
        main()
    captured = capsys.readouterr()
    assert captured.out == ("Valtteri Bottas      | MERCEDES                  | 1:12.434\n"
                            "Sebastian Vettel     | FERRARI                   | 1:04.415\n")
    with patch("sys.argv", ['main', '--files', folder_path, '--driver', 'Valtteri Bottas']):
        main()
    captured = capsys.readouterr()
    assert captured.out == "Valtteri Bottas      | MERCEDES                  | 1:12.434\n"


if __name__ == '__main__':
    main()




from datetime import datetime
import re
import argparse


def build_report(start_log_file, end_log_file, sort=None):
    """
    Build a report based on data from start and end log files.

    :param start_log_file: (str) the path to the start log file.
    :param end_log_file: (str) the path to the end log file.
    :param sort: (str, optional) the order in which the report should be sorted.
                 Accepted values 'desc'.  The default value is 'asc'
    :return: A dictionary containing the race results.
        In the format {driver_abbreviation: race_time}.
    """
    start_log = {}
    end_log = {}
    race_result = {}
    with open(start_log_file, 'r') as f:
        for line in f.readlines():
            key, value = re.findall(r"(.+?\D)(\d.+)", line.strip())[0]
            start_log[key] = value
    with open(end_log_file, 'r') as f:
        for line in f.readlines():
            key, value = re.findall(r"(.+?\D)(\d.+)", line.strip())[0]
            end_log[key] = value
    for key in start_log:
        start_time = datetime.strptime(start_log[key], '%Y-%m-%d_%H:%M:%S.%f')
        end_time = datetime.strptime(end_log[key], '%Y-%m-%d_%H:%M:%S.%f')
        result_time = abs(end_time - start_time)
        seconds_time = result_time.total_seconds()
        result_str = datetime.utcfromtimestamp(seconds_time).strftime('%M:%S.%f')
        race_result[key] = result_str.strip('0')
        if sort == 'desc':
            race_result = dict(sorted(race_result.items(), key=lambda item: item[1], reverse=True))
        else:
            race_result = dict(sorted(race_result.items(), key=lambda item: item[1]))

    return race_result


def print_report(abbreviations_file, build_result, driver_name=None):
    """
    Print report of a race or specific driver.

    :param abbreviations_file: (srt) the path to the file with the driver abbreviation data.
    :param build_result: (dict) dictionary with data on race results, with driver abbreviations
                        as key and race time as value.
    :param driver_name: (srt, optional) parameter to specify a specific driver for print race data.
    :return: - None
    """
    abbreviations = {}
    dict_result = {}
    with open(abbreviations_file, 'r', encoding='utf-8') as f:
        for line in f.readlines():
            key, value = line.strip().split('_', 1)
            abbreviations[key] = value
    for key in build_result:
        result = build_result[key]
        driver_team = abbreviations[key]
        driver, team = driver_team.split('_')
        if driver_name is None or driver_name == driver:
            dict_result[driver] = (team, result)
    if driver_name is None:
        driver_place = 0
        for driver, data in dict_result.items():
            driver_place += 1
            team, result = data
            if driver_place == 16:
                print('-' * 60)
            print(f"{driver:<20} | {team:<25} | {result}")
    else:
        team, result = dict_result[driver_name]
        print(f"{driver_name:<20} | {team:<25} | {result}")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument('--files', required=True, help='folder containing the data files')
    parser.add_argument('--driver', type=str, help='name of the driver')
    parser.add_argument('--asc', action='store_true', help='sort results in ascending order')
    parser.add_argument('--desc', action='store_true', help='sort results in descending order')
    args = parser.parse_args()

    folder_path = args.files
    driver_name = args.driver
    start_log_file = f'{folder_path}/start.log'
    end_log_file = f'{folder_path}/end.log'
    abbreviations_file = f'{folder_path}/abbreviations.txt'
    sorted_asc = build_report(start_log_file, end_log_file)
    if args.driver:
        print_report(abbreviations_file, sorted_asc, driver_name)
    elif args.desc:
        sorted_desc = build_report(start_log_file, end_log_file, sort='desc')
        print_report(abbreviations_file, sorted_desc)
    else:
        print_report(abbreviations_file, sorted_asc)


if __name__ == '__main__':
    main()
